/*
 * @file ps2gamelite.h
 *
 * @author Andy Lindsay
 *
 * @version 0.1
 *
 * @copyright
 * Copyright (C) Parallax, Inc. 2012. All Rights MIT Licensed.
 *
 * @brief Communicate with PlayStation 2 controller(s) and return
 * button and joystick data.
 */

#include "ps2gamelite.h"
#include "simpletools.h"

void ps2_game();

static volatile int poll[2] = {0x01, 0x42}; 

ps2game_t *ps2game_open(int dat, int cmd, int att, int clk)
{
  ps2game_t *device = malloc(sizeof(ps2game_t)+4);
  device->dat = dat;
  device->cmd = cmd;
  device->att = att;
  device->clk = clk;
  device->cog = 1 + cogstart(ps2_game, 
                             device, 
                             &device->stack, 
                             128*4);
  return device;
}


void ps2game_close(ps2game_t *device)
{
  if(device->cog > 0)
  {
    cogstop(device->cog - 1);
    device->cog = 0;
    free(device);
  }  
}

int ps2game_status(ps2game_t *device)
{
  return device->status;
}  

int ps2game_square(ps2game_t *device)
{
  return (device->thumbR & (1<<7)) ? 0 : 1;
}  

int ps2game_cross(ps2game_t *device)
{
  return (device->thumbR & (1<<6)) ? 0 : 1;
}  

int ps2game_circle(ps2game_t *device)
{
  return (device->thumbR & (1<<5)) ? 0 : 1;
}  

int ps2game_triangle(ps2game_t *device)
{
  return (device->thumbR & (1<<4)) ? 0 : 1;
}  

int ps2game_1R(ps2game_t *device)
{
  return (device->thumbR & (1<<3)) ? 0 : 1;
}  

int ps2game_1L(ps2game_t *device)
{
  return (device->thumbR & (1<<2)) ? 0 : 1;
}  

int ps2game_2R(ps2game_t *device)
{
  return (device->thumbR & (1<<1)) ? 0 : 1;
}  

int ps2game_2L(ps2game_t *device)
{
  return (device->thumbR & 1) ? 0 : 1;
}  

int ps2game_left(ps2game_t *device)
{
  return (device->thumbL & (1<<7)) ? 0 : 1;
}  

int ps2game_backward(ps2game_t *device)
{
  return (device->thumbL & (1<<6)) ? 0 : 1;
}  

int ps2game_right(ps2game_t *device)
{
  return (device->thumbL & (1<<5)) ? 0 : 1;
}  

int ps2game_forward(ps2game_t *device)
{
  return (device->thumbL & (1<<4)) ? 0 : 1;
}  

int ps2game_start(ps2game_t *device)
{
  return (device->thumbL & (1<<3)) ? 0 : 1;
}  

int ps2game_joyPressR(ps2game_t *device)
{
  return (device->thumbL & (1<<2)) ? 0 : 1;
}  

int ps2game_joyPressL(ps2game_t *device)
{
  return (device->thumbL & (1<<1)) ? 0 : 1;
}  

int ps2game_select(ps2game_t *device)
{
  return (device->thumbL & 1) ? 0 : 1;
}  

int ps2game_joyLX(ps2game_t *device)
{
  return device->joyLX;
}  

int ps2game_joyLY(ps2game_t *device)
{
  return device->joyLY;
}  

int ps2game_joyRX(ps2game_t *device)
{
  return device->joyRX;
}  

int ps2game_joyRY(ps2game_t *device)
{
  return device->joyRY;
}  


void ps2_game(ps2game_t *device)
{
  high(device->att);
  high(device->clk);
  low(device->cmd);
  pause(1);
  while(1)
  {
    int bit = poll[0] & 1;
    set_output(device->cmd, bit);
    low(device->att);
    low(device->clk);
    shift_out(device->cmd, device->clk, LSBFIRST, 8, (poll[0] >> 1) | (poll[1] & 0b10000000));
    shift_out(device->cmd, device->clk, LSBFIRST, 8, poll[1] >> 1);
    device->status = shift_in(device->dat, device->clk, LSBPRE, 8);
    device->thumbL = shift_in(device->dat, device->clk, LSBPRE, 8);
    device->thumbR = shift_in(device->dat, device->clk, LSBPRE, 8);
    device->joyRX  = shift_in(device->dat, device->clk, LSBPRE, 8);
    device->joyRY  = shift_in(device->dat, device->clk, LSBPRE, 8);
    device->joyLX  = shift_in(device->dat, device->clk, LSBPRE, 8);
    device->joyLY  = shift_in(device->dat, device->clk, LSBPRE, 8);
    high(device->clk);
    high(device->att);
    pause(10);
  }
}      

/**
 * TERMS OF USE: MIT License
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

